@@ -205,44 +205,67 @@ addWordToHash(struct hashValue *h, const Char *word)
 {
     uint32_t a = h->a, b = h->b, c = h->c;
 #ifdef SHORT_STRINGS
+#define GETK    if ((k = (uChar)*word++) == 0) break
 #ifdef WIDE_STRINGS
     assert(sizeof(Char) >= 4);
     while (1) {
 	unsigned k;
-	if ((k = (uChar)*word++) == 0) break; a += k;
-	if ((k = (uChar)*word++) == 0) break; b += k;
-	if ((k = (uChar)*word++) == 0) break; c += k;
+	GETK;
+	a += k;
+	GETK;
+	b += k;
+	GETK;
+	c += k;
 	mix(a, b, c);
     }
 #else
     assert(sizeof(Char) == 2);
     while (1) {
 	unsigned k;
-	if ((k = (uChar)*word++) == 0) break; a += k;
-	if ((k = (uChar)*word++) == 0) break; a += k << 16;
-	if ((k = (uChar)*word++) == 0) break; b += k;
-	if ((k = (uChar)*word++) == 0) break; b += k << 16;
-	if ((k = (uChar)*word++) == 0) break; c += k;
-	if ((k = (uChar)*word++) == 0) break; c += k << 16;
+	GETK;
+	a += k;
+	GETK;
+	a += k << 16;
+	GETK;
+	b += k;
+	GETK;
+	b += k << 16;
+	GETK;
+	c += k;
+	GETK;
+	c += k << 16;
 	mix(a, b, c);
     }
 #endif
 #else
     assert(sizeof(Char) == 1);
+#define GETK    if ((k = *word++) == 0) break
     while (1) {
 	unsigned k;
-	if ((k = *word++) == 0) break; a += k;
-	if ((k = *word++) == 0) break; a += k << 8;
-	if ((k = *word++) == 0) break; a += k << 16;
-	if ((k = *word++) == 0) break; a += k << 24;
-	if ((k = *word++) == 0) break; b += k;
-	if ((k = *word++) == 0) break; b += k << 8;
-	if ((k = *word++) == 0) break; b += k << 16;
-	if ((k = *word++) == 0) break; b += k << 24;
-	if ((k = *word++) == 0) break; c += k;
-	if ((k = *word++) == 0) break; c += k << 8;
-	if ((k = *word++) == 0) break; c += k << 16;
-	if ((k = *word++) == 0) break; c += k << 24;
+	GETK;
+	a += k;
+	GETK;
+	a += k << 8;
+	GETK;
+	a += k << 16;
+	GETK;
+	a += k << 24;
+	GETK;
+	b += k;
+	GETK;
+	b += k << 8;
+	GETK;
+	b += k << 16;
+	GETK;
+	b += k << 24;
+	GETK;
+	c += k;
+	GETK;
+	c += k << 8;
+	GETK;
+	c += k << 16;
+	GETK;
+	c += k << 24;
 	mix(a, b, c);
     }
 #endif
@@ -1054,15 +1077,15 @@ phist(struct Hist *hp, int hflg)
         output_raw = 1;
 	cleanup_push(&old_output_raw, output_raw_restore);
 	if (hflg & HIST_TIME)
-	    /* 
+	    /*
 	     * Make file entry with history time in format:
-	     * "+NNNNNNNNNN" (10 digits, left padded with ascii '0') 
+	     * "+NNNNNNNNNN" (10 digits, left padded with ascii '0')
 	     */
 
 	    xprintf("#+%010lu\n", (unsigned long)hp->Htime);
 
 	if (HistLit && hp->histline)
-	    xprintf("%S\n", hp->histline);
+	    xprintf("%" TCSH_S "\n", hp->histline);
 	else
 	    prlex(&hp->Hlex);
         cleanup_until(&old_output_raw);
@@ -1174,6 +1197,17 @@ dohist(Char **vp, struct command *c)
     }
 }
 
+void
+cleanhist(void)
+{
+    struct Hist *hp, *np;
+
+    for (hp = &Histlist; (np = hp->Hnext) != NULL;) {
+	if (np->Hnum != HIST_PURGE)
+	    return;
+	hremove(np), hfree(np);
+    }
+}
 
 char *
 fmthist(int fmt, ptr_t ptr)
@@ -1186,7 +1220,7 @@ fmthist(int fmt, ptr_t ptr)
 	return xasprintf("%6d", hp->Hnum);
     case 'R':
 	if (HistLit && hp->histline)
-	    return xasprintf("%S", hp->histline);
+	    return xasprintf("%" TCSH_S, hp->histline);
 	else {
 	    Char *istr, *ip;
 	    char *p;
@@ -1216,17 +1250,23 @@ dotlock_cleanup(void* lockpath)
 
 /* Save history before exiting the shell. */
 void
-rechist(Char *fname, int ref)
+rechist(Char *xfname, int ref)
 {
     Char    *snum, *rs;
-    int     fp, ftmp, oldidfds;
+    int     fp, ftmp, oldidfds, ophup_disabled;
     struct varent *shist;
     char path[MAXPATHLEN];
     struct stat st;
+    static Char *fname;
     static Char   *dumphist[] = {STRhistory, STRmhT, 0, 0};
 
-    if (fname == NULL && !ref) 
+    if (xfname == NULL && !ref)
 	return;
+
+    fname = xfname;
+    ophup_disabled = phup_disabled;
+    phup_disabled = 1;
+
     /*
      * If $savehist is just set, we use the value of $history
      * else we use the value in $savehist
@@ -1249,8 +1289,8 @@ rechist(Char *fname, int ref)
     /*
      * The 'savehist merge' feature is intended for an environment
      * with numerous shells being in simultaneous use. Imagine
-     * any kind of window system. All these shells 'share' the same 
-     * ~/.history file for recording their command line history. 
+     * any kind of window system. All these shells 'share' the same
+     * ~/.history file for recording their command line history.
      * We try to handle the case of multiple shells trying to merge
      * histories at the same time, by creating semi-unique filenames
      * and saving the history there first and then trying to rename
@@ -1261,7 +1301,7 @@ rechist(Char *fname, int ref)
 		 * by optional lock parameter to savehist.
      *
      * jw.
-     */ 
+     */
     /*
      * We need the didfds stuff before loadhist otherwise
      * exec in a script will fail to print if merge is set.
@@ -1292,19 +1332,20 @@ rechist(Char *fname, int ref)
 #endif
 	    }
 	    getexit(osetexit);
-	    if (setexit())
+	    if (setexit() == 0)
 		loadhist(fname, 1);
 	    resexit(osetexit);
 	}
     }
     rs = randsuf();
-    xsnprintf(path, sizeof(path), "%S.%S", fname, rs);
+    xsnprintf(path, sizeof(path), "%" TCSH_S ".%" TCSH_S, fname, rs);
     xfree(rs);
 
     fp = xcreat(path, 0600);
     if (fp == -1) {
 	didfds = oldidfds;
 	cleanup_until(fname);
+	phup_disabled = ophup_disabled;
 	return;
     }
     /* Try to preserve ownership and permissions of the original history file */
@@ -1326,9 +1367,10 @@ rechist(Char *fname, int ref)
 #ifndef WINNT_NATIVE
     (void)rename(path, short2str(fname));
 #else
-    (void)ReplaceFile( short2str(fname),path,NULL,0,NULL,NULL);
+    (void)ReplaceFile(short2str(fname), path, NULL, 0, NULL, NULL);
 #endif
     cleanup_until(fname);
+    phup_disabled = ophup_disabled;
 }
 
 
